///////////////////////////////////////////////////////////////////////////////
//  For project details and authors, refer to README and AUTHORS files
//
//  This file is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This file is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
//  To view the licence online, go to: http://www.gnu.org/copyleft/gpl.html
////////////////////////////////////////////////////////////////////////////////

#ifndef _LAYOUT_CANVAS_H_
#define _LAYOUT_CANVAS_H_


// Include wxWidgets' headers
#include <wx/wx.h> 
// CEGUI headers
#include <CEGUISystem.h>
#include "openglEditorRenderer.h"
#include "openglEditorTexture.h"

#include "wx/glcanvas.h"
#include "EditorDocument.h"


class EditorView;

/** Provides OpenGL output. This canvas is the actual visible part of our "view"
* on the "document". The CEGUI system must be initialised when rendering, which is verified.
*/

class LayoutGLCanvas : public wxGLCanvas
{
public:
    /** Constructor.*/
    LayoutGLCanvas( EditorView* view, wxWindow* parent, const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize );

    /** Destructor.*/
    ~LayoutGLCanvas();

    void setBackgroundColour( const wxColour& colour );
    void loadLayout( const wxString& layoutName,const wxString& schemeName,const wxString& fontName );

    void setView ( EditorView* aView )
    {
        m_view = aView;
    }

    /** Forces a reset of cegui.*/
    bool Reset();

    /** Forces a re-draw.*/
    void Render();

    /** Returns whether the document is still valid or not.*/
    bool isDocumentValid() const;

private:
    /** The view to which we are attached.*/
    EditorView* m_view;

    /** Pointer to cegui singleton.*/
    CEGUI::System* m_GUISystem;
    /** Pointer to cegui renderer.*/
    CEGUI::OpenGLRenderer* m_GUILayoutRenderer;

    // CEGUI widgets
    CEGUI::Window* m_layoutImage;

	CEGUI::Font* m_font;
	bool defaultAutoScaled;

	CEGUI::Scheme* m_scheme;
	const CEGUI::Image* m_mouseCursor;


    /** Current canvas width. Used alot so it's cached.*/
    float m_realWidth;

    /** Current canvas height. Used alot so it's cached.*/
    float m_realHeight;

    /** Last known mouseX position.*/
    long m_lastMouseX;

    /** Last known mouseY position.*/
    long m_lastMouseY;

    // Here we have scroll state variables; required because wxWidgets has
    // different behavior on different platforms for wxWindow scrollbars.
    // (wxGTK automated them without intervention, wxMSW did not).
    //! current horizontal scroll position
    int m_scrollPosX;
    //! horizontal size of the 'document' (zoomed image width)
    int m_scrollDocX;
    //! horizontal 'page' size (width of the canvas widow)
    int m_scrollPageX;
    //! current vertical scroll position
    int m_scrollPosY;
    //! vertical size of the 'document' (zoomed image height)
    int m_scrollDocY;
    //! vertical 'page' size (height of the canvas widow)
    int m_scrollPageY;

	double m_layoutZoomFactor;

    /** Renders the current CEGUI Layout.*/
    void renderCEGUILayout();

    bool resetCEGUI();

    //! Update the scrollbars
    void updateScrollbars();
    //! update scroll position of the image based on scrollbar values.
    void updateLayoutScrollPosition();
    //
    // wxWidgets bits below
    //
    /** Handled wxSizeEvent. Propagate the new resolution to the gui renderer.*/
    void OnResize( wxSizeEvent& event );

    /** Event handler: notifies us that the canvas needs a repaint. We render the current layout.*/
    void OnPaint( wxPaintEvent& event );

    /** Event handler: notifies us that the canvas needs a refresh of the background.
    * a good moment to draw the current background, if any.*/
    void OnErase( wxEraseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnMouseMotion( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnLeftDown( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnLeftUp( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnRightDown( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnRightUp( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnMiddleDown( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnMiddleUp( wxMouseEvent& event );

    /** Event handler: we handle mouse input here.*/
    void OnMouseWheel( wxMouseEvent& event );

    /** Event handler: handles key-down events. We use it to move the current selection.*/
    void OnKeyDown( wxKeyEvent& event );

    /** Event handler: stops moving the current selection.*/
    void OnKeyUp( wxKeyEvent& event );

    /** Event handler: handles key-down events. We use it to move the current selection.*/
    void OnChar( wxKeyEvent& event );


    /** Event handler: scrollbar changes */
    void OnScrollWin( wxScrollWinEvent& event );

	void OnLeaveWindow( wxMouseEvent& event);

	void OnEnterWindow( wxMouseEvent& event);


    DECLARE_EVENT_TABLE()
};

#endif // _LAYOUT_CANVAS_H_
